@@ -1,6 +1,6 @@
-'''
+"""
 Commands to manage locations (show, add, remove)
-'''
+"""
 
 from cleo import Command
 from rich.markdown import Markdown
@@ -10,12 +10,12 @@ from pollyxt_pipelines import locations, config
 
 
 class ShowLocations(Command):
-    '''
+    """
     Print all locations to the terminal
 
     locations-show
         {--details : Show all variables for each location}
-    '''
+    """
 
     def handle(self):
 
@@ -30,17 +30,17 @@ class ShowLocations(Command):
 
 
 class LocationPath(Command):
-    '''
+    """
     Prints the path to the file(s) where you can input a custom location
 
     locations-path
         {--user : Show only the user's config file (ignore the system file). This only makes sense on Linux.}
-    '''
+    """
 
     def handle(self):
         config_paths = config.config_paths()
 
-        if self.option('user'):
+        if self.option("user"):
             console.print(config_paths[-1].parent / "locations.ini")
         else:
             for path in config_paths:
